#ifndef __relorient5_c /* Copyright Bill.Triggs@inrialpes.fr */
#define __relorient5_c "$ Fri Jul 21 12:36:10 2000 $"
/** NO WARRANTY. Use & distribute freely. Fixes/feedback welcome. **/
/*------------------------------------------------------------------------
  Driver routine for 5 point relative orientation of calibrated cameras.
  The main routine for this is the multiresultant solver relorient5m(),
  but before calling this we perform an optional check for camera
  translations t near zero, |t|/z <<1 where z is the smallest
  point-camera distance. mod_t is the estimated (lower bound on) |t|/z.

  The |t|=0 approximate solution is included among the solutions
  returned iff mod_t <= max_t_zero, and the polynomial |t|!=0 solutions
  are included iff mod_t >= min_t_nonzero.
  ------------------------------------------------------------------------*/
#include "geom23.h"

extern void
relorient_approx_rot(double R[3][3], double *mod_t,
		     double y[/*n*/][3], double x[/*n*/][3], int n);
extern int
relorient5m(double qout[20][4], double Rtout[20][3][4], double dout[/*20*2*n*/], 
	    double y[/*n*/][3], double x[/*n*/][3], int n);

int
relorient5(double qout[21][4], double Rtout[21][3][4], double dout[/*21*2*n*/], 
	   double y[/*n*/][3], double x[/*n*/][3], int n,
	   double max_t_zero, double min_t_nonzero)
{
  double R1[3][3], mod_t = 1e10;
  int i,j, nsol = 0;
  
  /* If required, test for translations near zero. If the (unfortunately
   * lower) bound on |t|/z is below a threshold max_t_zero, we write the
   * approximate t=0 solution out. In this case, we make no attempt to
   * estimate either t itself, or the point-camera distances. The small
   * baseline might well make these estimates unreliable, and in any
   * case we could do no better than the nonlinear 5 point solution
   * below.
   */
  if (max_t_zero >= 0.0) {

    relorient_approx_rot(R1,&mod_t,y,x,n);

    if (mod_t <= max_t_zero) {
      /* Write out |t|=0 solution.
       */
      rot_to_quat(qout[0],R1);
      for (i=0; i<3; i++) {
	for (j=0; j<3; j++) Rtout[0][i][j] = R1[i][j];
	Rtout[0][i][W] = 0.0;
      }
      for (i=0; i<2*n; i++) dout[i] = 1.0; /* default depth value */
      qout++;
      Rtout++;
      dout += 2*n;
      nsol++;
    }
  }
  
  /* Append the polynomial 5 point solutions to the |t|=0 solution found
   * above (if any). This solution is ill-conditioned for |t|->0, so we
   * don't call the polynomial solution if the bound on |t|/z is smaller
   * than a threshold min_t_nonzero.
   */
  if (mod_t >= min_t_nonzero) {
    nsol += relorient5m(qout,Rtout,dout,y,x,n);
  }

  return nsol;
}


#endif /*__relorient5_c */
